import sys, string, os, win32com.client, win32file

def GetFCList(GP, Input_GDB):
    GP.Workspace = Input_GDB
    FCList = ""

    fds = GP.ListDatasets("*", "featuredataset")
    GMAP_Code = fds.Next() # Get the GMAP code from the feature dataset

    GP.Workspace = Input_GDB + "\\" + GMAP_Code

    fcs = GP.ListFeatureClasses() # Get the list of feature classes in the feature dataset
    fc = fcs.Next()

    i = 0
    while fc:
        i = i + 1
        fc = fcs.Next()
        
    j = 0
    fcs.Reset()

    fc = fcs.Next()    
    while fc: # For each feature class
        j = j + 1

        desc = GP.Describe(fc)
        
        if j == 1:
            FCList = fc.upper() + " (" + desc.AliasName + ")"
        elif i <> j:
            FCList = FCList + ", " + fc.upper() + " (" + desc.AliasName + ")"
        elif i == j and j <> 1:
            FCList = FCList + " and " + fc.upper() + " (" + desc.AliasName + ")"

        fc = fcs.Next()

    return FCList

def GetSDPATerm(GP, SDP):
    SDPA = ""

    if SDP == "US Geological Survey" or SDP == "U.S. Geological Survey":
        SDPA = "USGS"
    else:
        SDP_List = SDP.split(" ")
        for SDP_Part in SDP_List:
            if SDP_Part <> "of" and SDP_Part <> "and" and SDP_Part <> "the":
                if SDPA == "":
                    if SDP_Part == "Arizona":
                        SDPA = "AZ"
                    else:
                        SDPA = SDP_Part[0]
                else:
                    SDPA = SDPA + SDP_Part[0]
    return SDPA

def GetSDTTerm(GP, SDSN):
    if SDSN.startswith("GQ"):
        SDT = "Geologic Quadrangle (GQ) Map Series"
    elif SDSN.startswith("MF"):
        SDT = "Miscellaneous Field Investigations (MF) Series"
    elif SDSN.startswith("I"):
        SDT = "Miscellaneous Investigations (I) Series"
    elif SDSN.startswith("GM"):
        SDT = "Geologic Map Series"
    elif SDSN.startswith("OF"):
        SDT = "Open-File Report Series"
    elif SDSN.startswith("DVGQ"):
        SDT = "Digitally Vectorized Geologic Quadrangle Data"
    elif SDSN.startswith("HSDM"):
        SDT = "Geologic Hazards Section Digital Map"
    elif SDSN.startswith("DWM"):
        SDT = "Digital Web Map Series"
    elif SDSN.startswith("DAD"):
        SDT = "Digital Analytical Data"
    elif SDSN.startswith("PP"):
        SDT = "Professional Paper Series"   
    else:
        SDT = "NA"
        GP.AddWarning("***Search and replace term #SDT# could not be determined. Assumed to be NA")

    return SDT

def GetSDPPTerm(GP, State, SDPA):
    if SDPA == "USGS":
        SDPP = "Reston, Virginia"
    else:
        if State == "Colorado":
            SDPP = "Denver"
        elif State == "California":
            SDPP = "Sacramento"
        elif State == "Utah":
            SDPP = "Salt Lake City"
        elif State == "New Mexico":
            SDPP = "Socorro"
        elif State == "Montana":
            SDPP = "Butte"
        elif State == "Arizona":
            SDPP = "Tucson"
        elif State == "Kentucky":
            SDPP = "Lexington"
        elif State == "Maryland":
            SDPP = "Baltimore"
        elif State == "Washington":
            SDPP = "Olympia"
        elif State == "Virginia":
            SDPP = "Charlottesville"
        elif State == "Texas":
            SDPP = "Austin"
        elif State == "Wyoming":
            SDPP = "Laramie"
        elif State == "Idaho":
            SDPP = "Moscow"
        else:
            SDPP = "#SDPP#"
            GP.AddWarning("***Search and replace term #SDPP# could not be determined. Please replace manually")

        if SDPP <> "#SDPP#":
            SDPP = SDPP + ", " + State
            GP.AddWarning("***Search and replace term #SDPP# assumed to be " + SDPP)

    return SDPP    
            
def GetGeoExtent(GP, SR_Geo, fc, location):
    try:
        GP.Project_management(fc, location + "\\tempshape.shp", SR_Geo, "")
    except:
        GP.AddError("Unable to project input feature class " + fc)
        GP.AddMessage(GP.GetMessages())

    desc_geo = GP.Describe(location + "\\tempshape.shp")
    extent = desc_geo.Extent
    Geo_Extent = extent.split(" ")
    GP.Delete(location + "\\tempshape.shp")

    return Geo_Extent

def GetReplaceLine(metafile_line, outfile_line, replace_term, replace_list):
    
    replace_values = replace_list.split(";")
    if metafile_line.find("Keyword: " + replace_term) <> -1 or metafile_line.find("AlphaCode: " + replace_term) <> -1:
        for replace_value in replace_values:
            if replace_term == "#PC#":
                replace_value = replace_value.upper()
                
            if outfile_line.find(replace_term) <> -1:
                outfile_line = outfile_line.replace(replace_term, replace_value)
            else:
                outfile_line = outfile_line + metafile_line.replace(replace_term, replace_value)
    else:
        if replace_term == "#PC#":
            if outfile_line.find("Title:") <> -1:
                replace_value = replace_list.replace(";", ", ")
                outfile_line = outfile_line.replace(replace_term, replace_value.upper())
            else:
                outfile_line = outfile_line.replace(replace_term, replace_values[0].upper())
        elif replace_term == "#PN#" or replace_term == "#ST#" or replace_term == "#SDPLIST#":
            replace_value = replace_list
            while replace_value.count(";") > 0:
                if replace_value.count(";") == 1:
                    replace_value = replace_value.replace(";", " and ")
                else:
                    replace_value = replace_value.replace(";", ", ", 1)

            outfile_line = outfile_line.replace(replace_term, replace_value)
                
        else:
            outfile_line = outfile_line.replace(replace_term, replace_values[0])

    return outfile_line            

def GetMapInfo(GP, GMAP_Code, FindType, Map_Info_Table):
    MapInfo = ""
    if FindType == "MapName":
        FindField = "MAP_NAME"
    elif FindType == "MapTitle":
        FindField = "MAP_TITLE"
    elif FindType == "States":
        FindField = "STATES"
    elif FindType == "ParkCode":
        FindField = "PARK_CODE"
    elif FindType == "Credit":
        FindField = "CREDIT"
    else:
        GP.AddError("Unable to recognize find type: " + FindType + "!!!")  
    if GP.Exists(Map_Info_Table):
        rows = GP.SearchCursor(Map_Info_Table)
        row = rows.Next()
        found_flag = "n"
        GP.AddMessage("The GMAP_Code is " + GMAP_Code)
        
        while row:
            if GMAP_Code.lower() == row.GetValue("GMAP_CODE").lower():
                found_flag = "y"
                MapInfo = row.GetValue(FindField)
            row = rows.Next()
        del row, rows

        if found_flag == "n":
            GP.AddError("\n*****There is no match for " + GMAP_Code + " in " + Map_Info_Table + "!")
    else:
        GP.AddError("\n*****Unable to find " + Map_Info_Table + "!")

    return MapInfo

def GetParkNames(GP, PCs, Map_Info_Table):
    ParkNames = ""
    for ParkCode in PCs:
        ParkName = GetMapInfo(GP, ParkCode, "MapName", Map_Info_Table)
        if ParkNames == "":
            ParkNames = ParkName
        else:
            ParkNames = ParkNames + ";" + ParkName

    PNs = ParkNames.split(";")
    GP.AddWarning("\nThe primary park code is " +  PCs[0] + " and the primary park name is " + PNs[0])
    return ParkNames
        
                
            
