import sys, string, os, win32com.client
from GetReplaceTerms import *

def SearchAndReplacePark(GP, Input_GDB, meta_file, out_file, GMAP_Code, Park_Codes, Park_Names, States, UTM_Zone, SRName, SRCM,
                         Feat_Count, Geometry, SDTS_Type, Geo_Extent, FC_Alias, FCName, Date):
    # Create file handles for the output file and template metadata file
    metafileHandle = open(meta_file)
    outfileHandle = open(out_file, 'a')
    metafile_list = metafileHandle.readlines() # Create an array of the lines in the template
    FCList = GetFCList(GP, Input_GDB) # Get a list of all the feature classes in the list
    # Get the feature class abbreviation for cn
##    if meta_file.endswith("cn.txt"):
##        FC = FCName.replace(GMAP_Code.lower(), "")
##    else:
##        FC = ""

    # Create arrays for Park_Codes, Park_Names and States  
    PCs = Park_Codes.split(";")
    PNs = Park_Names.split(";")
    STs = States.split(";")
    State = States # Set State = States for later processing
    
    # Create a search and replace term that contains all the states in a list
    while State.count(";") > 0:
        if State.count(";") == 1:
            State = State.replace(";", " and ")
        else:
            State = State.replace(";", ", ", 1)

    # Look through each line in the template, searching and replacing all terms and writing output file...            
    for metafile_line in metafile_list:        
        outfile_line = metafile_line.replace("#C#", GMAP_Code.upper())
        outfile_line = outfile_line.replace("#LCC#", GMAP_Code.lower())
        outfile_line = GetReplaceLine(metafile_line, outfile_line, "#PC#", Park_Codes) # handle multiple park codes
        outfile_line = outfile_line.replace("#LCPC#", PCs[0].lower())
        outfile_line = GetReplaceLine(metafile_line, outfile_line, "#PN#", Park_Names) # handle multiple park names
        outfile_line = GetReplaceLine(metafile_line, outfile_line, "#ST#", States) # handle multiple states
        outfile_line = outfile_line.replace("#UTMZONE#", UTM_Zone)
        outfile_line = outfile_line.replace("#PROJ_CS#", SRName)
        outfile_line = outfile_line.replace("#UTM_CM#", SRCM)
        outfile_line = outfile_line.replace("#FEAT_COUNT#", Feat_Count)
        outfile_line = outfile_line.replace("#ESRI_GEOMETRY#", Geometry)
        outfile_line = outfile_line.replace("#SDTS_TYPE#", SDTS_Type)
        outfile_line = outfile_line.replace("#WEST#", Geo_Extent[0])
        outfile_line = outfile_line.replace("#EAST#", Geo_Extent[2])
        outfile_line = outfile_line.replace("#NORTH#", Geo_Extent[3])
        outfile_line = outfile_line.replace("#SOUTH#", Geo_Extent[1])
        outfile_line = outfile_line.replace("#DATE#", Date)
        if not FC_Alias == "":
            outfile_line = outfile_line.replace("#FC_ALIAS#", FC_Alias)
##        if meta_file.endswith("cn.txt"):
##            outfile_line = outfile_line.replace("#FCU#", FC.upper())
##            outfile_line = outfile_line.replace("#FCL#", FC.lower())
        outfile_line = outfile_line.replace("#MAP FEATURE CLASSES#", FCList)
        outfileHandle.write(outfile_line)

    metafileHandle.close()
    outfileHandle.close()
    GP.AddMessage("\n**Metadata file " + out_file + " successfully created**")
    
def SearchAndReplaceQuad(GP, out_file, Template_Location, Credit_list, LW_Title, DME_Title):
    # Set up a file handle for an input metadata file, read the lines, then close
    infileHandle = open(out_file)
    infile_list = infileHandle.readlines()
    infileHandle.close()

    # Set up an output file handle to write into...    
    outfileHandle = open(out_file, 'w')
    
    # Set up the contacts database
    contacts_db = Template_Location + "\\contacts.dbf"
    # Create an array from the list of Credits and set up CreditOrg variable
    #####Fix credits to look for Credit_Abbrev, if found, if not take Credit_list as is#########
    Credits = Credit_list.split(";")
    CreditOrg = ""
    CreditName = ""
    
    # If the contacts_db exists, get all of the credit organizations (CreditOrg) from the db    
    if GP.Exists(contacts_db):
        for Credit in Credits:
            found_flag = "n" # Set up a variable to discover whether a Credit is found our not
            rows = GP.SearchCursor(contacts_db)
            row = rows.Next()
            while row:
                if Credit == row.GetValue("CONTACTINI"):
                    found_flag = "y" # Set to 'y' because Credit was found
                    if CreditName == "":
                        CreditName = row.GetValue("CONTACTNAM")
                    elif CreditName.find(row.GetValue("CONTACTNAM")) == -1:
                        CreditName = CreditName + ";" + row.GetValue("CONTACTNAM")
                    if CreditOrg == "":
                        CreditOrg = row.GetValue("CONTACTORG")
                    elif CreditOrg.find(row.GetValue("CONTACTORG")) == -1:
                        CreditOrg = CreditOrg + ";" + row.GetValue("CONTACTORG")
                row = rows.Next()
            if found_flag == "n": # If the Credit is not found, send out an error...
                GP.AddError("Unable to find " + Credit + " in contacts.dbf. Please type initials EXACTLY as they appears in db or add!")

        # Create a list of the credit organizations formatted for #CREDIT# replace term
        while CreditOrg.count(";") > 0:
            if CreditOrg.count(";") == 1:
                CreditOrg = CreditOrg.replace(";", " and ")
            else:
                CreditOrg = CreditOrg.replace(";", ", ", 1)             
        # Create list of Credits formatted for #CREDIT# replace term
        while CreditName.count(";") > 0:
            if CreditName.count(";") == 1:
                CreditName = CreditName.replace(";", " and ")
            else:
                CreditName = CreditName.replace(";", ", ", 1)

        Credit = CreditName + " (" + CreditOrg + ")" # Create #CREDIT# replace term with credits and organizations        

        del row, rows
            
    else:
        GP.AddError("Unable to find contacts.dbf in " + Template_Location)
        Credit = ""

 

    # For each line in the input metadata file, search and replace terms and write to out file
    for infile_line in infile_list:
        if Credit <> "":
            outfile_line = infile_line.replace("#CREDIT#", Credit.replace("OMeara", "O\'Meara"))
        else:
            outfile_line = infile_line
            GP.AddError("Credit term not replaced!  Please review location of contacts.dbf!")

        outfile_line = outfile_line.replace("#LWTTL#", LW_Title)
        outfile_line = outfile_line.replace("#DMETTL#", DME_Title)
        outfileHandle.write(outfile_line)

    outfileHandle.close()
    
def SearchAndReplaceSource(GP, GDB_Location, Template_Location, out_file, fclist, States, Park_Table, Park_Code):
    # Set up file handles for the input metadata file and the file containing the source citation block
    infileHandle = open(out_file)
    if GP.Exists(Template_Location + "\\source_meta.txt"):
        sourcefileHandle = open(Template_Location + "\\source_meta.txt")
    else:
        GP.AddError("File source_meta.txt does not exist in " + Template_Location)

    #Read lines and close files for source citation block and input metadata file
    infile_list = infileHandle.readlines()
    infileHandle.close()
    sourcefile_list = sourcefileHandle.readlines()
    sourcefileHandle.close()
    
    outfileHandle = open(out_file, 'w') # Set up output metadata file handle
    
    # Split the feature class list into an array, as well as the input States
    fcs = fclist.split(";")
    STs = States.split(";")
    State = STs[0] # Set up the 'primary state', used for getting SDPP term

    # Create a list of all GMAP_IDs found in all feature classes in feature class list, multiple fcs for gremap
    GMAP_ID_List = ""
    for fc in fcs:
        rows = GP.SearchCursor(fc)
        row = rows.Next()
        
        while row:
            GMAP_ID = str(row.GetValue("GMAP_ID"))
            if GMAP_ID_List == "":
                GMAP_ID_List = GMAP_ID
            elif GMAP_ID_List.find(str(GMAP_ID)) == -1:
                GMAP_ID_List = GMAP_ID_List + ";" + GMAP_ID
            row = rows.Next()
        del row, rows
        
    GMAP_IDs = GMAP_ID_List.split(";")
        
    i = 1 # used for figuring out when to add a source citation block
    PAM = 0 # initial default for positional accuracy value
    src_type_list = "" # set up the source type list (paper, digital, etc.)
    sdp_list = "" # set up the source data publisher list
    
    for GMAP_ID in GMAP_IDs: # For each GMAP_ID
        maprows = GP.SearchCursor(Park_Table) # Set up search cursor on Park_Table
        maprow = maprows.Next()
        GP.AddMessage("Adding Source Citation Information for GMAP_ID " + GMAP_ID + "...")
        
        if i > 1: # re-read input file if first source citation already created and open out_file for writing
            infileHandle = open(out_file)
            infile_list = infileHandle.readlines()
            infileHandle.close()
            outfileHandle = open(out_file, 'w')
            
        while maprow:
            GMAP_ID2 = str(maprow.GetValue("GMAP_ID")) # Get the GMAP_ID value from the table
                
            if GMAP_ID == GMAP_ID2: # If the value is equal to the current value in GMAP_IDs, get replace terms
                SDA = maprow.GetValue("GMAP_AUTH")
                SDP = maprow.GetValue("GMAP_ORG")
                SDPA = GetSDPATerm(GP, SDP)
                SDPD = str(maprow.GetValue("GMAP_YEAR"))
                SDSD = str(maprow.GetValue("GMAP_SCALE"))

                if PAM < maprow.GetValue("GMAP_RMS"): # If the PAM is less than current RMS, then set to current RMS to get max
                    PAM = maprow.GetValue("GMAP_RMS")
                
                SDS = "1:" + SDSD[:len(SDSD)-3] + "," + SDSD[len(SDSD)-3:] # Get scale from scale denominator
                    
                SDSN = maprow.GetValue("GMAP_SER")
                SDT = GetSDTTerm(GP, SDSN)
                SDPP = GetSDPPTerm(GP, State, SDPA)
                SDTTL = maprow.GetValue("GMAP_TITLE")
                SRC_TYPE = str(maprow.GetValue("GMAP_TYPE"))
                
                # Create a list of all source types for each GMAP_ID associated with the feature class(es)
                if src_type_list == "":
                    src_type_list = SRC_TYPE
                elif src_type_list.find(SRC_TYPE) == -1:
                    src_type_list = src_type_list + ";" + SRC_TYPE

                # Create a list of all organizations contributing to map
                if sdp_list == "":
                    sdp_list = SDP
                elif sdp_list.find(SDP) == -1:
                    sdp_list = sdp_list + ";" + SDP

                # Depending on SRC_TYPE, open the source contribution replace term from a file
                err_flag = 0
                if SRC_TYPE == "digital data":
                    sourcetypefile = "source_cont_digital.txt"
                elif SRC_TYPE == "digital data and paper":
                    sourcetypefile = "source_cont_dig_paper.txt"
                else:
                    sourcetypefile = "source_cont_paper.txt"

                if GP.Exists(GDB_Location + "\\" + sourcetypefile):
                    sourcetypefile = GDB_Location + "\\" + sourcetypefile
                elif GP.Exists(Template_Location + "\\" + sourcetypefile):
                    sourcetypefile = Template_Location + "\\" + sourcetypefile
                else:
                    err_flag = 1
                    GP.AddError("Source contribution file " + sourcetypefile + " does not exist! Unable to replace source contribution!")

                if not err_flag == 1:
                    sourcetypeHandle = open(sourcetypefile)

                        
                    sourcetype_lines = sourcetypeHandle.readlines()
                    sourcetypeHandle.close()
                # Search and replace terms in input file
                for infile_line in infile_list:
                        if i == 1: # If first time, just search and replace
                            outfile_line = infile_line.replace("#SDA#", SDA)
                            outfile_line = outfile_line.replace("#SDP#", SDP)
                            outfile_line = outfile_line.replace("#SDPA#", SDPA)
                            outfile_line = outfile_line.replace("#SDPD#", SDPD)
                            outfile_line = outfile_line.replace("#SDS#", SDS)
                            outfile_line = outfile_line.replace("#SDSD#", SDSD)
                            outfile_line = outfile_line.replace("#SDSN#", SDSN)
                            outfile_line = outfile_line.replace("#SDT#", SDT)
                            outfile_line = outfile_line.replace("#SDPP#", SDPP)
                            outfile_line = outfile_line.replace("#SDTTL#", SDTTL)
                            outfile_line = outfile_line.replace("#SRC_TYPE#", SRC_TYPE)
                            outfile_line = outfile_line.replace("#SRC_CONT#", sourcetype_lines[0]) # First and only line in source cont file
                            outfileHandle.write(outfile_line)               
                        elif infile_line.find("Process_Step:") <> -1: # Otherwise read from source citation file
                            for sourcefile_line in sourcefile_list:
                                outfile_line = sourcefile_line.replace("#SDA#", SDA)
                                outfile_line = outfile_line.replace("#SDP#", SDP)
                                outfile_line = outfile_line.replace("#SDPA#", SDPA)
                                outfile_line = outfile_line.replace("#SDPD#", SDPD)
                                outfile_line = outfile_line.replace("#SDS#", SDS)
                                outfile_line = outfile_line.replace("#SDSD#", SDSD)
                                outfile_line = outfile_line.replace("#SDSN#", SDSN)
                                outfile_line = outfile_line.replace("#SDT#", SDT)
                                outfile_line = outfile_line.replace("#SDPP#", SDPP)
                                outfile_line = outfile_line.replace("#SDTTL#", SDTTL)
                                outfile_line = outfile_line.replace("#SRC_TYPE#", SRC_TYPE)
                                if err_flag == 0:
                                    outfile_line = outfile_line.replace("#SRC_CONT#", sourcetype_lines[0])
                                outfileHandle.write(outfile_line)
                        elif infile_line.find("Source_Used_Citation_Abbreviation") <> - 1:
                            outfile_line = infile_line.replace("\n","") + ", " + SDPA + " " + SDSN + "\n" # Create source used list
                            outfileHandle.write(outfile_line)
                        else:
                            outfileHandle.write(infile_line)
            maprow = maprows.Next()
            

            
        outfileHandle.close()

        i = i + 1

    # Re-open file and replace source data publisher list
    infileHandle = open(out_file)
    infile_list = infileHandle.readlines()
    infileHandle.close()
    outfileHandle = open(out_file, 'w')    
    for infile_line in infile_list:
        outfile_line = GetReplaceLine(infile_line, infile_line, "#SDPLIST#", sdp_list)
        outfileHandle.write(outfile_line)
        
    outfileHandle.close()
        
    # Replace process step based on the source types used in feature class(es)
    SearchAndReplaceProcess(GP, GDB_Location, Template_Location, out_file, src_type_list, str (PAM), Park_Code)

    del maprows, maprow
    #Add theme keywords to metadata record based on fc name and alias
    for fc in fcs:
        desc = GP.Describe(fc)
        Theme_Keyword = fc.upper() + " (" + desc.AliasName + ")"
        AddThemeKeyword(out_file, Template_Location, Theme_Keyword)
    
def AddPlaceKeyword(meta_file, Template_Location, Place_Keyword):
    infileHandle = open(meta_file)
    sourcefileHandle = open(Template_Location + "\\place_meta.txt")
    infile_list = infileHandle.readlines()
    infileHandle.close()
    
    sourcefile_list = sourcefileHandle.readlines()
    sourcefileHandle.close()
    
    outfileHandle = open(meta_file, 'w')

    for infile_line in infile_list:
        if infile_line.find("Place_Keyword: United States") <> -1:
            for sourcefile_line in sourcefile_list:
                outfile_line = sourcefile_line.replace("#PLACE#", Place_Keyword)
                outfileHandle.write(outfile_line)
        else:
            outfileHandle.write(infile_line)
            
    outfileHandle.close()
    
def AddThemeKeyword(meta_file, Template_Location, Theme_Keyword):
    infileHandle = open(meta_file)
    sourcefileHandle = open(Template_Location + "\\theme_meta.txt")
    infile_list = infileHandle.readlines()
    infileHandle.close()
    
    sourcefile_list = sourcefileHandle.readlines()
    sourcefileHandle.close()
    
    outfileHandle = open(meta_file, 'w')

    for infile_line in infile_list:
        if infile_line.find("Theme_Keyword: National Park Service") <> -1:
            for sourcefile_line in sourcefile_list:
                outfile_line = sourcefile_line.replace("#THEME#", Theme_Keyword)
                outfileHandle.write(outfile_line)
        else:
            outfileHandle.write(infile_line)
            
    outfileHandle.close()  
def SearchAndReplaceProcess(GP, GDB_Location, Template_Location, out_file, src_type_list, PAM, Park_Code):
    # Choose the process step and horiz pos accuracy report files based on source type
    if src_type_list.find("paper") <> -1 and src_type_list.find("digital") == -1:
        processfile = "process_dig.txt"
        horizaccfile = "horiz_pos_acc_paper.txt"
    elif src_type_list.find("paper") == -1 and src_type_list.find("digital") <> -1:
        processfile = "process_conv.txt"
        horizaccfile = "horiz_pos_acc_digital.txt"
    elif src_type_list.find("paper") <> -1 and src_type_list.find("digital") <> -1:
        processfile = "process_dig_conv.txt"
        horizaccfile = "horiz_pos_acc_dig_paper.txt"
    elif src_type_list.find("paper") == -1 and src_type_list.find("digital") == -1:
        processfile = "process_dig.txt"
        horizaccfile = "horiz_pos_acc_paper.txt"
        GP.AddWarning("Source type of paper or digital not found!  Using process step and horizontal accuracy report for paper.")
    else:
        GP.AddError("Source type in " + src_type_list + " not found")
        
    err_flag = 0

    if GP.Exists(GDB_Location + "\\" + processfile):
        processfile = GDB_Location + "\\" + processfile
    elif GP.Exists(Template_Location + "\\" + processfile):
        processfile = Template_Location + "\\" + processfile
    else: 
        GP.AddError("Process step file " + processfile + " does not exist! Unable to replace process step!")
        err_flag = 1
        
    if GP.Exists(GDB_Location + "\\" + horizaccfile):
        horizaccfile = GDB_Location + "\\" + horizaccfile
    elif GP.Exists(Template_Location + "\\" + horizaccfile):
        horizaccfile = Template_Location + "\\" + horizaccfile
    else: 
        GP.AddError("Horizontal accuracy file " + horizaccfile + " does not exist! Unable to replace horizontal accuracy report!")
        err_flag = 1
        
    # Open the processfile and horizaccfile if they exist, otherwise send an error to the geoprocessor
    if GP.Exists(processfile):
        processfileHandle = open(processfile)
        processfile_list = processfileHandle.read()
        processfileHandle.close()

    if GP.Exists(horizaccfile):
        horizaccfileHandle = open(horizaccfile)
        horizacc_list = horizaccfileHandle.read()
        horizaccfileHandle.close()
    
    # If no errors, then search and replace process and horizontal accuracy terms    
    if not err_flag == 1:
        infileHandle = open(out_file)
        infile_list = infileHandle.readlines()
        infileHandle.close()
        
        outfileHandle = open(out_file, 'w')

        for infile_line in infile_list:
            outfile_line = infile_line.replace("#PROCESS#", processfile_list) # first and only line in file
            outfile_line = outfile_line.replace("#ACC_REPORT#", horizacc_list)

            if PAM <> "0":
                outfile_line = outfile_line.replace("#PAM#", PAM)
            elif PAM == "0" and outfile_line.find("#PAM#") <> -1: # If there is a PAM term and PAM is 0, then add warning below
                GP.AddWarning("***The RMS value for a paper map was entered as 0 in the map table.  Please correct in table and metadata!***")

##            # Replace GREMAP term with text from file if the file is gremap, otherwise replace with nothing
##            if out_file.endswith("gremap.txt"):
##                outfile_line = outfile_line.replace("#GREMAP#", gremap_list[0])
##            else:
##                outfile_line = outfile_line.replace("#GREMAP#", "")
##

            outfileHandle.write(outfile_line)

        outfileHandle.close()

def SearchAndReplaceContact(GP, Template_Location, out_file, Proc_Contact, Meta_Contact):

    contacts_db = Template_Location + "\\contacts.dbf"
    
    # If contacts db exists then replace contact-related terms for process contact and metadata contact, otherwise add error
    if GP.Exists(contacts_db):
        rows = GP.SearchCursor(contacts_db)
        row = rows.Next()
        found_flag = "n"
        while row:
            con_type = "#P"
            for Contact in (Proc_Contact, Meta_Contact):
                infileHandle = open(out_file)

                infile_list = infileHandle.readlines()
                infileHandle.close()   
                if Contact == row.GetValue("CONTACTNAM"): # if the Contact is equal to current line, then get terms
                    found_flag = "y"
                    ContactOrg = row.GetValue("CONTACTORG")
                    ContactPos = row.GetValue("CONTACTPOS")
                    Address = row.GetValue("ADDRESS")
                    City = row.GetValue("CITY")
                    State = row.GetValue("STATE")
                    Zip = row.GetValue("ZIP")
                    Phone = row.GetValue("PHONE")
                    Fax = row.GetValue("FAX")
                    EMail = row.GetValue("EMAIL")
                    
                    outfileHandle = open(out_file, 'w')

                    # Search and replace terms...
                    for infile_line in infile_list:
                        outfile_line = infile_line.replace(con_type + "CONTACT#", Contact.replace("OMeara", "O\'Meara"))
                        outfile_line = outfile_line.replace(con_type + "CONT_ORG#", ContactOrg)
                        outfile_line = outfile_line.replace(con_type + "CONT_POS#", ContactPos)
                        outfile_line = outfile_line.replace(con_type + "ADDRESS#", Address)
                        outfile_line = outfile_line.replace(con_type + "CITY#", City)
                        outfile_line = outfile_line.replace(con_type + "STATE#", State)
                        outfile_line = outfile_line.replace(con_type + "ZIP#", Zip)
                        outfile_line = outfile_line.replace(con_type + "PHONE#", Phone)
                        outfile_line = outfile_line.replace(con_type + "FAX#", Fax)
                        outfile_line = outfile_line.replace(con_type + "EMAIL#", EMail)
                        outfileHandle.write(outfile_line)
                        
                    outfileHandle.close()
                con_type = "#M"
            row = rows.Next()
        del row, rows
        
        if found_flag == "n":
            GP.AddError("\n***Unable to find " + Contact + " in contacts.dbf.  Please check spelling of name!***\n")
    else:
        GP.AddError("\ncontacts.dbf file could not be found in " + Template_Location)
